'                        Acknowledgements
' Jeffrey Richter:
' For his original implementation of a layout algorithm which positions
' controls in a window based on a set of rules.

' Notice: Portions Copyright  1995 Jeffrey Richter

' Uncomment to load the debugging version
%DEBUG_LAYOUT = 1

' Action to perform
%STRETCH = 0        ' Metric should be stretched
%MOVE    = 1        ' Control should be moved
%VCENTER = 2        ' Vertically center control/group
%HCENTER = 3        ' Horizontally center control/group

' Part to act on or relative to
%LEFT    = 0        ' Left side of control
%TOP     = 1        ' Top side of control
%RIGHT   = 2        ' Right side of control
%BOTTOM  = 3        ' Bottom side of control
%WIDTH   = 4        ' Width of control
%HEIGHT  = 5        ' Height of control
%CONTROL = 6        ' Group(one control)
%WIDGET  = %CONTROL ' Widget and Control are synonymous
%GROUP   = 7        ' Group(two controls acting as limits for a range of controls)

' Scale mode
%SCALEMODE_NONE     = -1
%SCALEMODE_DPI      = 0
%SCALEMODE_FONT     = 1
%SCALEMODE_DIALOG   = 2

' Special value indicating that the window
' being acted relative to is the form
%FORM      = &H7FFFFFFE&

' Special value indicating that the maximum
' size of a window is umlimited
%NOLIMIT            = 0

#If Not %Def(%DEBUG_LAYOUT)
Declare Function Layout_AddRule Lib "layout32.dll" Alias "Layout_AddRule" _
  ( _
           ByVal hWnd           As DWord, _ ' [in] Handle of top-level window
           ByVal lAction        As Long, _  ' [in] Action to perform
           ByVal lActOnPart     As Long, _  ' [in] Part of control to act on
           ByVal lActOnControl  As Long, _  ' [in] Control(s) to act on(loword=first, hiword=last)
  Optional ByVal lRelToPart     As Long, _  ' [in] Part of control to act relative to
  Optional ByVal lRelToControl  As Long, _  ' [in] Control to act relative to
  Optional ByRef lOffset        As Long _   ' [in] Amount to offset acted-on part from relative-to part
  ) As Long

Declare Function Layout_AddLimit Lib "layout32.dll" Alias "Layout_AddLimit" _
  ( _
           ByVal hWnd       As DWord, _ ' [in] Handle of top-level window
           ByVal lControl   As Long, _  ' [in] Control(s) to act on(loword=first, hiword=last)
  Optional ByVal lMinWidth  As Long, _  ' [in] Minimum width of control
  Optional ByVal lMinHeight As Long, _  ' [in] Minimum height of control
  Optional ByVal lMaxWidth  As Long, _  ' [in] Maximum width of control
  Optional ByVal lMaxHeight As Long _   ' [in] Maximum height of control
  ) As Long

Declare Function Layout_Clear Lib "layout32.dll" Alias "Layout_Clear"_
  ( _
  ByVal hWnd  As DWord _ ' [in] Handle of top-level window
  ) As Long

Declare Function Layout_SetScaleMode Lib "layout32.dll" Alias "Layout_SetScaleMode" _
  ( _
  ByVal hWnd        As DWord, _ ' [in] Handle of top-level window
  ByVal lScaleMode  As Long _   ' [in] New scale mode
  ) As Long

Declare Function Layout_SetResolution Lib "layout32.dll" Alias "Layout_SetResolution" _
  ( _
  ByVal hWnd      As DWord, _ ' [in] Handle of top-level window
  ByVal lHorzRes  As Long, _  ' [in] Horizontal screen resolution(DPI) or character width
  ByVal lVertRes  As Long _   ' [in] Vertical screen resolution(DPI) or character height
  ) As Long

Declare Function Layout_HideSizeGrip Lib "layout32.dll" Alias "Layout_HideSizeGrip" _
  ( _
  ByVal hWnd  As DWord, _ ' [in] Handle of top-level window
  ByVal fHide As Long _   ' [in] If True, size-box is hidden when window is maximized
  ) As Long
#Else
Declare Function Layout_AddRule Lib "layout32_dbg.dll" Alias "Layout_AddRule" _
  ( _
           ByVal hWnd           As DWord, _ ' [in] Handle of top-level window
           ByVal lAction        As Long, _  ' [in] Action to perform
           ByVal lActOnPart     As Long, _  ' [in] Part of control to act on
           ByVal lActOnControl  As Long, _  ' [in] Control(s) to act on(loword=first, hiword=last)
  Optional ByVal lRelToPart     As Long, _  ' [in] Part of control to act relative to
  Optional ByVal lRelToControl  As Long, _  ' [in] Control to act relative to
  Optional ByRef lOffset        As Long _   ' [in] Amount to offset acted-on part from relative-to part
  ) As Long

Declare Function Layout_AddLimit Lib "layout32_dbg.dll" Alias "Layout_AddLimit" _
  ( _
           ByVal hWnd       As DWord, _ ' [in] Handle of top-level window
           ByVal lControl   As Long, _  ' [in] Control(s) to act on(loword=first, hiword=last)
  Optional ByVal lMinWidth  As Long, _  ' [in] Minimum width of control
  Optional ByVal lMinHeight As Long, _  ' [in] Minimum height of control
  Optional ByVal lMaxWidth  As Long, _  ' [in] Maximum width of control
  Optional ByVal lMaxHeight As Long _   ' [in] Maximum height of control
  ) As Long

Declare Function Layout_Clear Lib "layout32_dbg.dll" Alias "Layout_Clear"_
  ( _
  ByVal hWnd  As DWord _ ' [in] Handle of top-level window
  ) As Long

Declare Function Layout_SetScaleMode Lib "layout32_dbg.dll" Alias "Layout_SetScaleMode" _
  ( _
  ByVal hWnd        As DWord, _ ' [in] Handle of top-level window
  ByVal lScaleMode  As Long _   ' [in] New scale mode
  ) As Long

Declare Function Layout_SetResolution Lib "layout32_dbg.dll" Alias "Layout_SetResolution" _
  ( _
  ByVal hWnd      As DWord, _ ' [in] Handle of top-level window
  ByVal lHorzRes  As Long, _  ' [in] Horizontal screen resolution(DPI) or character width
  ByVal lVertRes  As Long _   ' [in] Vertical screen resolution(DPI) or character height
  ) As Long

Declare Function Layout_HideSizeGrip Lib "layout32_dbg.dll" Alias "Layout_HideSizeGrip" _
  ( _
  ByVal hWnd  As DWord, _ ' [in] Handle of top-level window
  ByVal fHide As Long _   ' [in] If True, size-box is hidden when window is maximized
  ) As Long
#EndIf
